@@ -9,13 +9,16 @@ import functools
 import itertools
 import math
 import numbers
+from packaging import version
 import random
 import re
 import sys
 import warnings
 
+import pytz
 from sentinels import NOTHING
 
+import mongomock
 from mongomock import command_cursor
 from mongomock import filtering
 from mongomock import helpers
@@ -89,6 +92,7 @@ projection_operators = [
 date_operators = [
     '$dateFromString',
     '$dateToString',
+    '$dateFromParts',
     '$dayOfMonth',
     '$dayOfWeek',
     '$dayOfYear',
@@ -108,7 +112,6 @@ array_operators = [
     '$concatArrays',
     '$filter',
     '$indexOfArray',
-    '$isArray',
     '$map',
     '$range',
     '$reduce',
@@ -163,6 +166,10 @@ type_convertion_operators = [
     '$arrayToObject',
     '$objectToArray',
 ]
+type_operators = [
+    '$isNumber',
+    '$isArray',
+]
 
 
 def _avg_operation(values):
@@ -256,6 +263,8 @@ class _Parser(object):
                 return self._handle_string_operator(k, v)
             if k in type_convertion_operators:
                 return self._handle_type_convertion_operator(k, v)
+            if k in type_operators:
+                return self._handle_type_operator(k, v)
             if k in boolean_operators:
                 return self._handle_boolean_operator(k, v)
             if k in text_search_operators + projection_operators + object_operators:
@@ -569,7 +578,13 @@ class _Parser(object):
             'pipeline, it is currently not implemented  in Mongomock.' % operator)
 
     def _handle_date_operator(self, operator, values):
-        out_value = self.parse(values)
+        if isinstance(values, dict) and values.keys() == {'date', 'timezone'}:
+            value = self.parse(values['date'])
+            target_tz = pytz.timezone(values['timezone'])
+            out_value = value.replace(tzinfo=pytz.utc).astimezone(target_tz)
+        else:
+            out_value = self.parse(values)
+
         if operator == '$dayOfYear':
             return out_value.timetuple().tm_yday
         if operator == '$dayOfMonth':
@@ -620,6 +635,42 @@ class _Parser(object):
                     ' in Mongomock.'
                 )
             return out_value['date'].strftime(out_value['format'])
+        if operator == '$dateFromParts':
+            if not isinstance(out_value, dict):
+                raise OperationFailure(
+                    f'{operator} operator must correspond a dict '
+                    'that has "year" or "isoWeekYear" field.'
+                )
+            if len(set(out_value) & {'year', 'isoWeekYear'}) != 1:
+                raise OperationFailure(
+                    f'{operator} operator must correspond a dict '
+                    'that has "year" or "isoWeekYear" field.'
+                )
+            for field in ('isoWeekYear', 'isoWeek', 'isoDayOfWeek', 'timezone'):
+                if field in out_value:
+                    raise NotImplementedError(
+                        f'Although {field} is a valid field for the '
+                        f'{operator} operator, it is currently not implemented '
+                        'in Mongomock.'
+                    )
+
+            year = out_value['year']
+            month = out_value.get('month', 1) or 1
+            day = out_value.get('day', 1) or 1
+            hour = out_value.get('hour', 0) or 0
+            minute = out_value.get('minute', 0) or 0
+            second = out_value.get('second', 0) or 0
+            millisecond = out_value.get('millisecond', 0) or 0
+
+            return datetime.datetime(
+                year=year,
+                month=month,
+                day=day,
+                hour=hour,
+                minute=minute,
+                second=second,
+                microsecond=millisecond
+            )
 
         raise NotImplementedError(
             "Although '%s' is a valid date operator for the "
@@ -876,15 +927,43 @@ class _Parser(object):
             'aggregation pipeline, it is currently not implemented '
             'in Mongomock.' % operator)
 
+    def _handle_type_operator(self, operator, values):
+        # Document: https://docs.mongodb.com/manual/reference/operator/aggregation/isNumber/
+        if operator == '$isNumber':
+            try:
+                parsed = self.parse(values)
+            except KeyError:
+                return False
+            return False if isinstance(parsed, bool) else isinstance(parsed, numbers.Number)
+
+        # Document: https://docs.mongodb.com/manual/reference/operator/aggregation/isArray/
+        if operator == '$isArray':
+            try:
+                parsed = self.parse(values)
+            except KeyError:
+                return False
+            return isinstance(parsed, (tuple, list))
+
+        raise NotImplementedError(  # pragma: no cover
+            "Although '%s' is a valid type operator for the aggregation pipeline, it is currently "
+            'not implemented in Mongomock.' % operator)
+
     def _handle_conditional_operator(self, operator, values):
         if operator == '$ifNull':
-            field, fallback = values
-            try:
-                out_value = self.parse(field)
-                if out_value is not None:
-                    return out_value
-            except KeyError:
-                pass
+            fields = values[:-1]
+            if len(fields) > 1 and version.parse(mongomock.SERVER_VERSION) <= version.parse('4.4'):
+                raise OperationFailure(
+                    '$ifNull supports only one input expression '
+                    ' in MongoDB v4.4 and lower'
+                )
+            fallback = values[-1]
+            for field in fields:
+                try:
+                    out_value = self.parse(field)
+                    if out_value is not None:
+                        return out_value
+                except KeyError:
+                    pass
             return self.parse(fallback)
         if operator == '$cond':
             if isinstance(values, list):
